@@ -1,9 +1,11 @@
 from .guard import Scanner, Guard, InputGuard, OutputGuard, ScannerResult
+from ._exceptions import APIStatusError
 
 __all__ = [
     'Scanner',
     'Guard', 
     'InputGuard', 
     'OutputGuard',
-    'ScannerResult'
+    'ScannerResult',
+    'APIStatusError'
     ]
