@@ -7,6 +7,7 @@ import asyncio
 import inspect
 from typing import Any, Awaitable, Callable, Dict, Optional, Union
 import httpx
+from ._exceptions import InternalServerError, PermissionDeniedError, BadRequestError, NotFoundError, ConflictError, RateLimitError, AuthenticationError, TsGuardError, UnprocessableEntityError
 
 Callback = Callable[[ScannerResult], Union[None, Awaitable[None]]]
 
@@ -79,7 +80,7 @@ class Guard:
     Exception
         For failed API requests or file operations.
     """
-    def __init__(self, API_KEY, PROJECT_ID, remote_addr=REMOTE_TS_API_ADDRESS, fail_fast=True):
+    def __init__(self, API_KEY, PROJECT_ID=None, remote_addr=REMOTE_TS_API_ADDRESS, fail_fast=True):
         """        
         scan_mode : str
             "input": analyzes prompts sent to the llm
@@ -110,7 +111,7 @@ class Guard:
         Args:
             scanner (Scanner): The scanner instance to be added.
         """
-        assert isinstance(scanner, Scanner), "Scanner must be an instance of Scanner class."
+        assert isinstance(scanner, Scanner), f"Scanner must be an instance of Scanner class, got {type(scanner)}"
         if self.scanners is None:
             self.scanners = []
         self.scanners.append(scanner)
@@ -127,6 +128,64 @@ class Guard:
             if hasattr(scanner, '_requires_input_prompt') and scanner._requires_input_prompt:
                 requires_input.add(scanner.__class__.__name__)
         return scanners_dict, requires_input
+    
+    def get_available_scanners(self) -> Dict[str, Any]:
+        """
+        Retrieves all scanners available to the API key 
+        
+        Returns:
+            Dict[str, Any]: A dictionary containing:
+            - input_scanners (List[str]): List of available input scanners in "type:tag" format
+            - output_scanners (List[str]): List of available output scanners in "type:tag" format  
+            - plan_name (str): The name of the current subscription plan
+            
+            
+        Raises:
+            AuthenticationError: If the API key is invalid or inactive.
+            PermissionDeniedError: If the subscription is not active.
+            InternalServerError: For server errors.
+        """
+        url = f'{self.remote_addr}/scanners'
+        
+        response = requests.get(
+            url,
+            headers={
+                'X-API-Key': self.API_KEY
+            }
+        )
+        
+        response_json = self._safe_json_parse(response)
+        
+        if response.status_code == 401:
+            raise AuthenticationError(
+                message=self._get_error_message("Authentication failed", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 403:
+            raise PermissionDeniedError(
+                message=self._get_error_message("Permission denied", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 404:
+            raise NotFoundError(
+                message=self._get_error_message("Resource not found", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code >= 500:
+            raise InternalServerError(
+                message=f"Server error: {response.status_code}",
+                response=response,
+                body=response_json
+            )
+        
+        
+        return response_json
 
     def _prepare_request_json(self, prompt, project_id, scanners: Dict, output = None, multimodal=False):
         req_dict = {
@@ -145,6 +204,27 @@ class Guard:
             req_dict["output"] = output
         return req_dict
     
+    def _safe_json_parse(self, response):
+        """Safely parse JSON response, return None if parsing fails."""
+        try:
+            return response.json()
+        except (ValueError, json.JSONDecodeError):
+            return {"error": "Invalid JSON response", "text": response.text[:500]}
+        
+    def _get_error_message(self, default_message, response_body):
+        """Extract error message from response body"""
+        if response_body and isinstance(response_body, dict):
+            message = response_body.get('message')
+            details = response_body.get('details')
+            
+            if message and details:
+                return f"{message} - {details}"
+            elif message:
+                return message
+            elif details:
+                return f"{default_message} - {details}"
+        return default_message  
+    
     def _make_request(self, data, url: str, files: List[str]=None, async_mode: bool = False, callback: Optional[Callback] = None):
         if files is not None and len(files) > 0:
             return self._make_multimodal_request(data, url, files, async_mode=async_mode, callback=callback)
@@ -160,7 +240,6 @@ class Guard:
                     'x-api-key': self.API_KEY,
                     'Content-Type': 'application/json'
                 },
-                async_mode=True,
                 callback=callback
             )
         response = requests.post(
@@ -171,10 +250,65 @@ class Guard:
             },
             data=data
         )
-        if response.status_code != 200:
-            raise Exception(f"Request failed with status code {response.status_code}")
+        response_json = self._safe_json_parse(response)
+        if response.status_code == 400:
+            raise BadRequestError(
+                message=self._get_error_message("Bad request", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 401:
+            raise AuthenticationError(
+                message=self._get_error_message("Authentication failed", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 403:
+            raise PermissionDeniedError(
+                message=self._get_error_message("Permission denied", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 404:
+            raise NotFoundError(
+                message=self._get_error_message("Resource not found", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 409:
+            raise ConflictError(
+                message=self._get_error_message("Conflict error", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 422:
+            raise UnprocessableEntityError(
+                message=self._get_error_message("Unprocessable entity", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 429:
+            raise RateLimitError(
+                message=self._get_error_message("Rate limit exceeded", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code >= 500:
+            raise InternalServerError(
+                message=f"Server error: {response.status_code}",
+                response=response,
+                body=response_json
+            )
+
+        
         
-        response_json = response.json()
         return ScannerResult(**response_json)
     
     def _make_multimodal_request(self, data, url: str, files: List[str], async_mode: bool = False, callback: Optional[Callback] = None):
@@ -217,10 +351,63 @@ class Guard:
             files=payload_files
         )
         
-        if response.status_code != 200:
-            raise Exception(f"Request failed with status code {response.status_code}")
+        response_json = self._safe_json_parse(response)
+        if response.status_code == 400:
+            raise BadRequestError(
+                message=self._get_error_message("Bad request", response_json),
+                response=response,
+                body=response_json
+            )
         
-        response_json = response.json()
+        if response.status_code == 401:
+            raise AuthenticationError(
+                message=self._get_error_message("Authentication failed", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 403:
+            raise PermissionDeniedError(
+                message=self._get_error_message("Permission denied", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 404:
+            raise NotFoundError(
+                message=self._get_error_message("Resource not found", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 409:
+            raise ConflictError(
+                message=self._get_error_message("Conflict error", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 422:
+            raise UnprocessableEntityError(
+                message=self._get_error_message("Unprocessable entity", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code == 429:
+            raise RateLimitError(
+                message=self._get_error_message("Rate limit exceeded", response_json),
+                response=response,
+                body=response_json
+            )
+        
+        if response.status_code >= 500:
+            raise InternalServerError(
+                message=f"Server error: {response.status_code}",
+                response=response,
+                body=response_json
+            )
+
         return ScannerResult(**response_json)
     
     def _request_api_async_mode(self,
@@ -353,7 +540,7 @@ class InputGuard(Guard):
                 AssertionError: If files is not a list of strings.
     """
 
-    def __init__(self, API_KEY, PROJECT_ID, remote_addr=REMOTE_TS_API_ADDRESS, fail_fast=True):
+    def __init__(self, API_KEY=None, PROJECT_ID=None, remote_addr=None, fail_fast=True):
         """
         Initializes the Guard class with the provided API key, project ID, and optional parameters.
         Args:
@@ -362,7 +549,15 @@ class InputGuard(Guard):
             remote_addr (str, optional): The remote address of the TS API. Defaults to REMOTE_TS_API_ADDRESS.
             fail_fast (bool, optional): If True, initialization will fail immediately on errors. Defaults to True.
         """
-
+        if API_KEY is None:
+            API_KEY = os.environ.get("TEST_SAVANT_API_KEY")
+        if PROJECT_ID is None:
+            PROJECT_ID = os.environ.get("TEST_SAVANT_PROJECT_ID", None)
+        if API_KEY is None:
+            raise ValueError("API_KEY must be provided either as an argument or via the TEST_SAVANT_API_KEY environment variable.")
+        
+        if remote_addr is None:
+            remote_addr = os.environ.get("TEST_SAVANT_REMOTE_ADDR", REMOTE_TS_API_ADDRESS)
         super().__init__(API_KEY, PROJECT_ID, remote_addr,fail_fast=fail_fast)
         self.remote_addr = remote_addr
 
@@ -386,6 +581,7 @@ class InputGuard(Guard):
             AssertionError: If files is not a list of strings.
         """
 
+        assert isinstance(prompt, str), f"Prompt must be a string, got {type(prompt)}"
         if not self.scanners:
             raise ValueError("No scanners have been added.")
         
@@ -429,7 +625,7 @@ class OutputGuard(InputGuard):
                 ValueError: If no scanners have been added or if required input prompt is missing.
     """
 
-    def __init__(self, API_KEY, PROJECT_ID, remote_addr=REMOTE_TS_API_ADDRESS, fail_fast=True):
+    def __init__(self, API_KEY=None, PROJECT_ID=None, remote_addr=None, fail_fast=True):
         """
         Initializes the Guard class with API credentials and configuration options.
         Args:
@@ -438,7 +634,14 @@ class OutputGuard(InputGuard):
             remote_addr (str, optional): The remote API address. Defaults to REMOTE_TS_API_ADDRESS.
             fail_fast (bool, optional): If True, initialization will fail immediately on errors. Defaults to True.
         """
-
+        if API_KEY is None:
+            API_KEY = os.environ.get("TEST_SAVANT_API_KEY")
+        if PROJECT_ID is None:
+            PROJECT_ID = os.environ.get("TEST_SAVANT_PROJECT_ID", None)
+        if API_KEY is None:
+            raise ValueError("API_KEY must be provided either as an argument or via the TEST_SAVANT_API_KEY environment variable.")
+        if remote_addr is None:
+            remote_addr = os.environ.get("TEST_SAVANT_REMOTE_ADDR", REMOTE_TS_API_ADDRESS)
         super().__init__(API_KEY, PROJECT_ID, remote_addr,fail_fast=fail_fast)
         self.remote_addr = remote_addr
         
