@@ -8,7 +8,7 @@ class Anonymize(Scanner):
         For all available tags, check: https://docs.testsavant.ai/docs/v1/python/input-scanners
     """
     threshold: Optional[confloat(ge=0.0, le=1.0)] = None
-    use_faker: Optional[bool] = None
+    use_faker: Optional[bool] = False
     preamble: Optional[str] = None
     tag: Literal["base"]
     result: Optional[ScannerResult] = None
@@ -29,7 +29,23 @@ class Anonymize(Scanner):
             "US_SSN_RE",
             "USERNAME",
             "PASSWORD",
-        ]]]
+        ]]] = [
+            "CREDIT_CARD",
+            "CRYPTO",
+            "EMAIL_ADDRESS",
+            "IBAN_CODE",
+            "IP_ADDRESS",
+            "PERSON",
+            "PHONE_NUMBER",
+            "US_SSN",
+            "US_BANK_NUMBER",
+            "CREDIT_CARD_RE",
+            "UUID",
+            "EMAIL_ADDRESS_RE",
+            "US_SSN_RE",
+            "USERNAME",
+            "PASSWORD",
+        ]
     allowed_names: Optional[List[str]] = None
 
 
